import pygame
from pygame.mixer import music
from pygame.locals import *

from OpenGL.GL import *

import flyer, world, quest, questmanager, gui, config
from data import get_path

class Scene (object):
	music = None
	volume = 1.0
	fade_speed = 0.01

	def __init__(self, manager):
		self.manager = manager

	def init(self):
		if self.music:
			music.load(get_path(self.music))
			music.set_volume(self.volume)
			music.play(-1)
	
	def fade_music(self):
		vol = music.get_volume()
		speed = self.fade_speed
		if vol - self.volume > speed:
			music.set_volume(vol - speed)
		elif self.volume - vol > speed:
			music.set_volume(vol + speed)
		else:
			music.set_volume(self.volume)

	def think(self):
		ddpygame.event.clear()
	
	def render(self, surf):
		pass

def draw_shader(r, g, b, amount):
		glLoadIdentity()
		glPushAttrib(GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT | GL_CURRENT_BIT)
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glDisable(GL_FOG)
		glDisable(GL_DEPTH_TEST)
		glDisable(GL_TEXTURE_2D)
		glColor4f(r, g, b, amount)
		glBegin(GL_QUADS)
		glVertex3f(-2, -1, -2)
		glVertex3f(-2, 1, -2)
		glVertex3f(2, 1, -2)
		glVertex3f(2, -1, -2)
		glEnd()
		glPopAttrib()

class StartScene (Scene):
	music = 'supersole - horizon blue.mp3'
	volume = 0.0

	def think(self):
		return 'title', ()

class TitleScene (Scene):
	volume = 0.7

	def init(self):
		Scene.init(self)

		
		self.env = world.Endless()
		self.env.compile()
		self.env.setup()

		self.title = gui.Text('Flight', size=192, color=(255, 255, 255))
		self.anykey = gui.Text('Press any key to start.', size=24, color=(192, 192, 192))

		self.sw, self.sh = config.screen_size
	
		self.time = 0.
		self.start = None
	
	def think(self):
		self.fade_music()

		for ev in pygame.event.get():
			if ev.type == KEYDOWN:
				if ev.key == K_ESCAPE:
					return 'exit', ()
				else:
					self.start = self.time

		self.env.think()
		self.time += .01

		if self.start is not None and self.time - self.start > 1:
			return 'flight', ()

	def render(self):
		self.env.render()

		if self.time < 1.:
			draw_shader(0, 0, 0, 1 - self.time)



		glPushAttrib(GL_CURRENT_BIT)
		glColor4f(1, 1, 1, max(0, min(1, self.time * 1.5 - 1.8)))
		self.title.render(10, 10)
		glColor4f(1, 1, 1, max(0, min(1, self.time * 1.5 - 2.5)))
		self.anykey.render(self.sw - self.anykey.w - 10, self.sh - self.anykey.h - 10)

		glPopAttrib()

		if self.start is not None:
			draw_shader(1, 1, 1, self.time - self.start)

class FlightScene (Scene):
	volume = 0.2
	
	def init(self):
		Scene.init(self)

		questmanager.set_quest(quest.Intro())

		self.flyer = flyer.Flyer(world.World.spawn_pos)
		self.world = world.World(self.flyer)

		self.kstate = dict.fromkeys([K_LEFT, K_RIGHT, K_UP, K_DOWN])

		self.world.setup()
		self.world.compile()

	def think(self):
		self.fade_music()

		kstate = self.kstate

		for ev in pygame.event.get():
			if ev.type == KEYDOWN and ev.key == K_ESCAPE:
				self.world.stop()
				return 'pause', (self,)

			if ev.type in [KEYUP, KEYDOWN]:
				kstate[ev.key] = ev.type == KEYDOWN

		h_force = .3
		v_force = 2.
		throttle = 10.
		
		flyer = self.flyer

		if flyer.alive:
			if kstate[K_LEFT]:
				flyer.wings[0].body.addRelForce((0, 0, -h_force))
				flyer.wings[1].body.addRelForce((0, 0, h_force))
			if kstate[K_RIGHT]:
				flyer.wings[0].body.addRelForce((0, 0, h_force))
				flyer.wings[1].body.addRelForce((0, 0, -h_force))
			if kstate[K_UP]:
				flyer.wings[2].body.addRelForce((0, 0, v_force))
			if kstate[K_DOWN]:
				flyer.wings[2].body.addRelForce((0, 0, -v_force))

			#if kstate[K_SPACE]:
			#	flyer.wings[0].body.addRelForce((throttle, 0, 0))
			#	flyer.wings[1].body.addRelForce((throttle, 0, 0))

		else:
			if flyer.pos.z < self.world.deepline:
				self.world.stop()
				return 'gameover', ()

		self.world.think()
		questmanager.get_quest().think()
		gui.messenger.think()

	def render(self):
		self.world.render()
		gui.messenger.render()



class ShadeScene (Scene):
	shade = .6

	def __init__(self, manager, prev_scene):
		Scene.__init__(self, manager)
		self.prev_scene = prev_scene

	def init(self):
		Scene.init(self)
		self.counter = 0.
	
	def render(self):
		if self.counter < self.shade:
			self.counter += 0.05

		self.prev_scene.render()
		draw_shader(0, 0, 0, self.counter)

class PauseScene (ShadeScene):
	def init(self):
		ShadeScene.init(self)
		self.paused = gui.Text('PAUSED', size=64, color=(255, 255, 255))
		self.help = gui.Text('Press Q to quit, or Esc to continue.', size=24, color=(192, 192, 192))
		self.sw, self.sh = config.screen_size

	def think(self):
		for ev in pygame.event.get():
			if ev.type == KEYDOWN:
				if ev.key == K_ESCAPE:
					return self.prev_scene, ()
				if ev.key == K_q:
					return 'title', ()
	
	def render(self):
		ShadeScene.render(self)
		self.paused.render(10, 10)
		self.help.render(self.sw - self.help.w - 10, self.sh - self.help.h - 10)

class GameOverScene (Scene):
	def init(self):
		self.sw, self.sh = config.screen_size
		self.over = gui.Text('Game over', size=64, color=(255, 255, 255))
		self.tragic = gui.Text('You have drowned without knowing what happend to you.', size=24, color=(255, 255, 255))
		self.anykey = gui.Text('Press any key to continue.', size=24, color=(128, 128, 128))

		self.counter = -5.
		glClearColor(0, 0, 0, 1)
	
	def think(self):
		for ev in pygame.event.get():
			if ev.type == KEYDOWN:
				return 'title', ()

	def render(self):
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT)

		self.counter += .05

		glColor4f(1, 1, 1, min(1., self.counter + 5.))
		self.over.render((self.sw - self.over.w) / 2, self.sh / 3)
		self.tragic.render((self.sw - self.tragic.w) / 2, self.sh / 2)
		
		glColor4f(1, 1, 1, max(0, min(self.counter, 1.)))
		self.anykey.render(self.sw - self.anykey.w - 10, self.sh - self.anykey.h - 10)
		glPopAttrib()

